@@ -1,9 +1,9 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
  *******************************************************************************
- * Copyright (C) 2002-2010, International Business Machines Corporation and    *
- * others. All Rights Reserved.                                                *
- *******************************************************************************
- *
+ * Copyright (C) 2002-2014, International Business Machines Corporation and
+ * others. All Rights Reserved.
  *******************************************************************************
  */
 
@@ -17,6 +17,10 @@ import java.nio.charset.CharsetEncoder;
 import java.nio.charset.CoderResult;
 import java.nio.charset.CodingErrorAction;
 import java.util.Iterator;
+import java.util.List;
+
+import org.junit.Test;
+import org.junit.runner.RunWith;
 
 import com.ibm.icu.charset.CharsetCallback;
 import com.ibm.icu.charset.CharsetDecoderICU;
@@ -24,27 +28,34 @@ import com.ibm.icu.charset.CharsetEncoderICU;
 import com.ibm.icu.charset.CharsetICU;
 import com.ibm.icu.charset.CharsetProviderICU;
 import com.ibm.icu.dev.test.ModuleTest;
+import com.ibm.icu.dev.test.ModuleTest.TestDataPair;
 import com.ibm.icu.dev.test.TestDataModule.DataMap;
+import com.ibm.icu.dev.test.TestDataModule.TestData;
+import com.ibm.icu.dev.test.TestFmwk;
 import com.ibm.icu.impl.ICUResourceBundle;
 import com.ibm.icu.text.UnicodeSet;
 
+import junitparams.JUnitParamsRunner;
+import junitparams.Parameters;
+
 /**
- * This maps to convtest.c which tests the test file for data-driven conversion tests. 
- * 
+ * This maps to convtest.c which tests the test file for data-driven conversion tests.
+ *
  */
-public class TestConversion extends ModuleTest {
+@RunWith(JUnitParamsRunner.class)
+public class TestConversion extends TestFmwk {
     /**
      * This maps to the C struct of conversion case in convtest.h that stores the
      * data for a conversion test
-     * 
+     *
      */
     private class ConversionCase {
-        int caseNr;                                             // testcase index   
+        int caseNr;                                             // testcase index
         String option = null;                                   // callback options
         CodingErrorAction cbErrorAction = null;                 // callback action type
         CharBuffer toUnicodeResult = null;
         ByteBuffer fromUnicodeResult = null;
-        
+
         // data retrieved from a test case conversion.txt
         String charset;                                         // charset
         String unicode;                                         // unicode string
@@ -53,74 +64,79 @@ public class TestConversion extends ModuleTest {
         boolean finalFlush;                                     // flush
         boolean fallbacks;                                      // fallback
         String outErrorCode;                                    // errorCode
-        String cbopt;                                           // callback 
-        
+        String cbopt;                                           // callback
+
         // TestGetUnicodeSet variables
         String map;
         String mapnot;
         int which;
-        
+
         // CharsetCallback encoder and decoder
         CharsetCallback.Decoder cbDecoder = null;
         CharsetCallback.Encoder cbEncoder = null;
-        
+
         String caseNrAsString() {
             return "[" + caseNr + "]";
         }
     }
 
+    /* In the data-driven conversion test, converters that are not available in
+     * ICU4J are marked with the following leading symbol.
+     */
+    private static final char UNSUPPORTED_CHARSET_SYMBOL = '+';
+
     // public methods --------------------------------------------------------
 
-    public static void main(String[] args) throws Exception {
-        new TestConversion().run(args);
+    public TestConversion() {
     }
 
-    public TestConversion() {
-        super("com/ibm/icu/dev/data/testdata/", "conversion");
+    @SuppressWarnings("unused")
+    private List<TestDataPair> getTestData() throws Exception {
+        return ModuleTest.getTestData("com/ibm/icu/dev/data/testdata/", "conversion");
     }
 
     /*
      * This method maps to the convtest.cpp runIndexedTest() method to run each
      * type of conversion.
      */
-    public void processModules() {
-        try {
-            int testFromUnicode = 0;
-            int testToUnicode = 0;
-            String testName = t.getName().toString();
+    @Test
+    @Parameters(method="getTestData")
+    public void conversionTest(TestDataPair pair) {
+        TestData td = pair.td;
+        //DataMap settings = pair.dm;
 
-            // Iterate through and get each of the test case to process
-            for (Iterator iter = t.getDataIterator(); iter.hasNext();) {
-                DataMap testcase = (DataMap) iter.next();
+        int testFromUnicode = 0;
+        int testToUnicode = 0;
+        String testName = td.getName().toString();
 
-                if (testName.equalsIgnoreCase("toUnicode")) {
-                    TestToUnicode(testcase, testToUnicode);
-                    testToUnicode++;
+        // Iterate through and get each of the test case to process
+        for (Iterator iter = td.getDataIterator(); iter.hasNext();) {
+            DataMap testcase = (DataMap) iter.next();
 
-                } else if (testName.equalsIgnoreCase("fromUnicode")) {
-                    TestFromUnicode(testcase, testFromUnicode);
-                    testFromUnicode++;
-                } else if (testName.equalsIgnoreCase("getUnicodeSet")) {
-                    TestGetUnicodeSet(testcase);
-                } else {
-                    warnln("Could not load the test cases for conversion");
-                    continue;
-                }
+            if (testName.equalsIgnoreCase("toUnicode")) {
+                TestToUnicode(testcase, testToUnicode);
+                testToUnicode++;
+
+            } else if (testName.equalsIgnoreCase("fromUnicode")) {
+                TestFromUnicode(testcase, testFromUnicode);
+                testFromUnicode++;
+            } else if (testName.equalsIgnoreCase("getUnicodeSet")) {
+                TestGetUnicodeSet(testcase);
+            } else {
+                warnln("Could not load the test cases for conversion");
+                //                    continue;
             }
-        } catch (Exception e) {
-            e.printStackTrace();
         }
-
     }
 
     // private methods -------------------------------------------------------
 
-    
-    // fromUnicode test worker functions --------------------------------------- 
+
+    // fromUnicode test worker functions ---------------------------------------
     private void TestFromUnicode(DataMap testcase, int caseNr) {
-        
+
         ConversionCase cc = new ConversionCase();
-        
+
         try {
             // retrieve test case data
             cc.caseNr = caseNr;
@@ -139,6 +155,25 @@ public class TestConversion extends ModuleTest {
             return;
         }
 
+        /*
+         * Skip the following data driven converter tests.
+         * These tests were added to the data driven conversion test in ICU
+         * to test direct-from-UTF-8 m:n Unicode:charset conversion.
+         * This feature is not in ICU4J.
+         * See #9601
+         */
+        String [] testsToSkip = {
+                "*test2"
+        };
+        for (int i = 0; i < testsToSkip.length; i++) {
+            if (cc.charset.equals(testsToSkip[i])) {
+                logln("");
+                logln("Skipping: " + cc.charset);
+                logln("...............................................");
+                return;
+            }
+        }
+
         // ----for debugging only
         logln("");
         logln("TestFromUnicode[" + caseNr + "] " + cc.charset + " ");
@@ -180,7 +215,7 @@ public class TestConversion extends ModuleTest {
                 break;
             }
 
-            // check for any options for the callback value -- 
+            // check for any options for the callback value --
             cc.option = cc.cbErrorAction == null ? cc.cbopt : cc.cbopt
                     .substring(1);
             if (cc.option == null) {
@@ -190,9 +225,8 @@ public class TestConversion extends ModuleTest {
         FromUnicodeCase(cc);
     }
 
-    
-    private void FromUnicodeCase(ConversionCase cc) {
 
+    private void FromUnicodeCase(ConversionCase cc) {
         // create charset encoder for conversion test
         CharsetProviderICU provider = new CharsetProviderICU();
         CharsetEncoder encoder = null;
@@ -203,26 +237,30 @@ public class TestConversion extends ModuleTest {
                     ? (Charset) provider.charsetForName(cc.charset.substring(1),
                         "com/ibm/icu/dev/data/testdata", this.getClass().getClassLoader())
                     : (Charset) provider.charsetForName(cc.charset);
-            encoder = (CharsetEncoder) charset.newEncoder();
-            encoder.onMalformedInput(CodingErrorAction.REPLACE);
-            encoder.onUnmappableCharacter(CodingErrorAction.REPLACE);
-            if (encoder instanceof CharsetEncoderICU) {
-                ((CharsetEncoderICU)encoder).setFallbackUsed(cc.fallbacks);
-                if (((CharsetEncoderICU)encoder).isFallbackUsed() != cc.fallbacks) {
-                    errln("Fallback could not be set for " + cc.charset);
+            if (charset != null) {
+                encoder = charset.newEncoder();
+                encoder.onMalformedInput(CodingErrorAction.REPLACE);
+                encoder.onUnmappableCharacter(CodingErrorAction.REPLACE);
+                if (encoder instanceof CharsetEncoderICU) {
+                    ((CharsetEncoderICU)encoder).setFallbackUsed(cc.fallbacks);
+                    if (((CharsetEncoderICU)encoder).isFallbackUsed() != cc.fallbacks) {
+                        errln("Fallback could not be set for " + cc.charset);
+                    }
                 }
             }
-            
         } catch (Exception e) {
-            if (skipIfBeforeICU(4,5,0)) { // TIME BOMB
-                logln("Skipping test:(" + cc.charset + ") due to ICU Charset not supported at this time");
+            encoder = null;
+        }
+        if (encoder == null) {
+            if (cc.charset.charAt(0) == UNSUPPORTED_CHARSET_SYMBOL) {
+                logln("Skipping test:(" + cc.charset.substring(1) + ") due to ICU Charset not supported at this time");
             } else {
                 errln(cc.charset + " was not found");
             }
             return;
         }
-        
-        // set the callback for the encoder 
+
+        // set the callback for the encoder
         if (cc.cbErrorAction != null) {
             if (cc.cbEncoder != null) {
                 ((CharsetEncoderICU)encoder).setFromUCallback(CoderResult.malformedForLength(1), cc.cbEncoder, cc.option);
@@ -238,7 +276,7 @@ public class TestConversion extends ModuleTest {
             }
 
             // if callback action is replace,
-          //   and there is a subchar
+            //   and there is a subchar
             // replace the decoder's default replacement value
             // if substring, skip test due to current api not supporting
             // substring
@@ -326,7 +364,7 @@ public class TestConversion extends ModuleTest {
             break;
         }
     }
-    
+
     private int stepFromUnicode(ConversionCase cc, CharsetEncoder encoder, int step) {
         if (step < 0) {
             errln("Negative step size, test internal error.");
@@ -349,7 +387,7 @@ public class TestConversion extends ModuleTest {
             currentSourceLimit = sourceLen;
             currentTargetLimit = targetLen;
         }
-        
+
         CoderResult cr = null;
 
         for (;;) {
@@ -357,7 +395,7 @@ public class TestConversion extends ModuleTest {
             target.limit(currentTargetLimit);
 
             cr = encoder.encode(source, target, currentSourceLimit == sourceLen);
-            
+
             if (cr.isUnderflow()) {
                 if (currentSourceLimit == sourceLen) {
                     if (target.position() == cc.bytes.limit()) {
@@ -395,11 +433,11 @@ public class TestConversion extends ModuleTest {
             }
 
         }
-        
+
         cc.fromUnicodeResult = target;
         return target.position();
     }
-    
+
     private boolean checkFromUnicode(ConversionCase cc, int resultLength) {
         return checkResultsFromUnicode(cc, cc.bytes, cc.fromUnicodeResult);
     }
@@ -409,7 +447,7 @@ public class TestConversion extends ModuleTest {
     private void TestToUnicode(DataMap testcase, int caseNr) {
         // create Conversion case to store the test case data
         ConversionCase cc = new ConversionCase();
-        
+
         try {
             // retrieve test case data
             cc.caseNr = caseNr;
@@ -490,14 +528,18 @@ public class TestConversion extends ModuleTest {
                     ? (Charset) provider.charsetForName(cc.charset.substring(1),
                         "com/ibm/icu/dev/data/testdata", this.getClass().getClassLoader())
                     : (Charset) provider.charsetForName(cc.charset);
-            decoder = (CharsetDecoder) charset.newDecoder();
-            decoder.onMalformedInput(CodingErrorAction.REPLACE);
-            decoder.onUnmappableCharacter(CodingErrorAction.REPLACE);
-
+            if (charset != null) {
+                decoder = charset.newDecoder();
+                decoder.onMalformedInput(CodingErrorAction.REPLACE);
+                decoder.onUnmappableCharacter(CodingErrorAction.REPLACE);
+            }
         } catch (Exception e) {
             // TODO implement loading of test data.
-            if (skipIfBeforeICU(4,5,0)) {
-                logln("Skipping test:(" + cc.charset + ") due to ICU Charset not supported at this time");
+            decoder = null;
+        }
+        if (decoder == null) {
+            if (cc.charset.charAt(0) == UNSUPPORTED_CHARSET_SYMBOL) {
+                logln("Skipping test:(" + cc.charset.substring(1) + ") due to ICU Charset not supported at this time");
             } else {
                 errln(cc.charset + " was not found");
             }
@@ -546,13 +588,13 @@ public class TestConversion extends ModuleTest {
             }
         }
 
-        //      Check the step to unicode    
+        //      Check the step to unicode
         boolean ok;
         int resultLength;
 
         String steps[][] = { { "0", "bulk" }, // must be first for offsets to be checked
                 { "1", "step=1" }, { "3", "step=3" }, { "7", "step=7" } };
-        /* TODO: currently not supported test steps, getNext API is not supported for now  
+        /* TODO: currently not supported test steps, getNext API is not supported for now
          { "-1", "getNext" },
          { "-2", "toU(bulk)+getNext" },
          { "-3", "getNext+toU(bulk)" },
@@ -571,7 +613,7 @@ public class TestConversion extends ModuleTest {
                 continue;
             }
             logln("Testing step:[" + step + "]");
-            
+
             try {
                 resultLength = stepToUnicode(cc, decoder, step);
                 ok = checkToUnicode(cc, resultLength);
@@ -624,7 +666,7 @@ public class TestConversion extends ModuleTest {
 
 
 
-    
+
     private int stepToUnicode(ConversionCase cc, CharsetDecoder decoder,
             int step)
 
@@ -660,14 +702,14 @@ public class TestConversion extends ModuleTest {
                     target.limit(target.capacity());
                     flush = cc.finalFlush;
                 }
-                // convert 
+                // convert
                 CoderResult cr = null;
                 if (source.hasRemaining()) {
 
                     cr = decoder.decode(source, target, flush);
                     // check pointers and errors
                     if (cr.isOverflow()) {
-                        // the partial target is filled, set a new limit, 
+                        // the partial target is filled, set a new limit,
                         oStep = (target.position() + step);
                         target.limit((oStep < target.capacity()) ? oStep
                                 : target.capacity());
@@ -691,9 +733,11 @@ public class TestConversion extends ModuleTest {
 
                         cr = decoder.decode(source, target, true);
 
-                        //due to limitation of the API we need to check for target limit for expected 
-                        if (target.limit() != cc.unicode.length()) {
-                            target.limit(cc.unicode.length());
+                        //due to limitation of the API we need to check for target limit for expected
+                        if (target.position() != cc.unicode.length()) {
+                            if (target.limit() != cc.unicode.length()) {
+                                target.limit(cc.unicode.length());
+                            }
                             cr = decoder.flush(target);
                             if (cr.isError()) {
                                 errln("Flush operation failed");
@@ -737,7 +781,7 @@ public class TestConversion extends ModuleTest {
                         if (cr.isOverflow()) {
 
                             if (target.limit() >= target.capacity()) {
-                                // target has reached its limit, an error occurred 
+                                // target has reached its limit, an error occurred
                                 logln("UnExpected error: Target Buffer is larger than capacity");
                                 break;
                             } else {
@@ -797,7 +841,7 @@ public class TestConversion extends ModuleTest {
                     }
                     CoderResult cr = decoder.decode(source, target, source
                             .limit() == sourceLen);
-                    // check pointers and errors 
+                    // check pointers and errors
                     if (cr.isOverflow()) {
                         // one character has been consumed
                         if (target.limit() >= target.capacity()) {
@@ -824,12 +868,12 @@ public class TestConversion extends ModuleTest {
     }
 
 
-   
+
     private boolean checkToUnicode(ConversionCase cc, int resultLength) {
         return checkResultsToUnicode(cc, cc.unicode, cc.toUnicodeResult);
     }
 
-    
+
     private void TestGetUnicodeSet(DataMap testcase) {
         /*
          * charset - will be opened, and ucnv_getUnicodeSet() called on it //
@@ -838,14 +882,14 @@ public class TestConversion extends ModuleTest {
          * returned set // which - numeric UConverterUnicodeSet value Headers {
          * "charset", "map", "mapnot", "which" }
          */
-       
-        
+
+
         // retrieve test case data
         ConversionCase cc = new ConversionCase();
         CharsetProviderICU provider = new CharsetProviderICU();
         CharsetICU charset  ;
-       
-             
+
+
         UnicodeSet mapset = new UnicodeSet();
         UnicodeSet mapnotset = new UnicodeSet();
         UnicodeSet unicodeset = new UnicodeSet();
@@ -855,87 +899,87 @@ public class TestConversion extends ModuleTest {
         cc.map = ((ICUResourceBundle) testcase.getObject("map")).getString();
         cc.mapnot = ((ICUResourceBundle) testcase.getObject("mapnot"))
                 .getString();
-        
-     
+
+
         cc.which = ((ICUResourceBundle) testcase.getObject("which")).getInt(); // only checking for ROUNDTRIP_SET
-        
+
         // ----for debugging only
         logln("");
         logln("TestGetUnicodeSet[" + cc.charset + "] ");
         logln("...............................................");
-        
+
         try{
-           // if cc.charset starts with '*', obtain it from com/ibm/icu/dev/data/testdata
-           charset = (cc.charset != null && cc.charset.length() > 0 && cc.charset.charAt(0) == '*')
+            // if cc.charset starts with '*', obtain it from com/ibm/icu/dev/data/testdata
+            charset = (cc.charset != null && cc.charset.length() > 0 && cc.charset.charAt(0) == '*')
                     ? (CharsetICU) provider.charsetForName(cc.charset.substring(1),
-                        "com/ibm/icu/dev/data/testdata", this.getClass().getClassLoader())
-                    : (CharsetICU) provider.charsetForName(cc.charset);
-           
-           //checking for converter that are not supported at this point        
-           try{
-               if(charset.name()=="BOCU-1" ||charset.name()== "SCSU"|| charset.name()=="lmbcs1" || charset.name()== "lmbcs2" ||
-                      charset.name()== "lmbcs3" || charset.name()== "lmbcs4" || charset.name()=="lmbcs5" || charset.name()=="lmbcs6" ||
-                      charset.name()== "lmbcs8" || charset.name()=="lmbcs11" || charset.name()=="lmbcs16" || charset.name()=="lmbcs17" || 
-                      charset.name()=="lmbcs18"|| charset.name()=="lmbcs19"){
-                   
-                   logln("Converter not supported at this point :" +charset.displayName());
-                   return;
-               }
-                             
-               if(cc.which==1){
-                   logln("Fallback set not supported at this point for converter : "+charset.displayName());
-                  return;
-               }
-               
-           }catch(Exception e){
-               return;
-           }
-           
-           mapset.clear();
-           mapnotset.clear();
-                   
-           mapset.applyPattern(cc.map,false);
-           mapnotset.applyPattern(cc.mapnot,false);
-           
-           charset.getUnicodeSet(unicodeset, cc.which);
-           UnicodeSet diffset = new UnicodeSet();
-           
-           //are there items that must be in unicodeset but are not?           
-           (diffset = mapset).removeAll(unicodeset);
-           if(!diffset.isEmpty()){
-               StringBuffer s = new StringBuffer(diffset.toPattern(true));
-               if(s.length()>100){
-                   s.replace(0, 0x7fffffff, ellipsis);
-               }
-               errln("error in missing items - conversion/getUnicodeSet test case "+cc.charset + "\n" + s.toString());
-           }
-           
-          //are the items that must not be in unicodeset but are?
-           (diffset=mapnotset).retainAll(unicodeset);
-           if(!diffset.isEmpty()){
-               StringBuffer s = new StringBuffer(diffset.toPattern(true));
-               if(s.length()>100){
-                   s.replace(0, 0x7fffffff, ellipsis);
-               }
-               errln("contains unexpected items - conversion/getUnicodeSet test case "+cc.charset + "\n" + s.toString());
-           }
-         } catch (Exception e) {
-             errln("getUnicodeSet returned an error code");
-             errln("ErrorCode expected is: " + cc.outErrorCode);
-             errln("Error Result is: " + e.toString());
-             return;
-         }
+                            "com/ibm/icu/dev/data/testdata", this.getClass().getClassLoader())
+                            : (CharsetICU) provider.charsetForName(cc.charset);
+
+                    //checking for converter that are not supported at this point
+                    try{
+                        if(charset==null ||
+                                charset.name()=="BOCU-1" ||charset.name()== "SCSU"|| charset.name()=="lmbcs1" || charset.name()== "lmbcs2" ||
+                                charset.name()== "lmbcs3" || charset.name()== "lmbcs4" || charset.name()=="lmbcs5" || charset.name()=="lmbcs6" ||
+                                charset.name()== "lmbcs8" || charset.name()=="lmbcs11" || charset.name()=="lmbcs16" || charset.name()=="lmbcs17" ||
+                                charset.name()=="lmbcs18"|| charset.name()=="lmbcs19"){
+                            logln("Converter not supported at this point :" + cc.charset);
+                            return;
+                        }
+
+                        if(cc.which==1){
+                            logln("Fallback set not supported at this point for converter : "+charset.displayName());
+                            return;
+                        }
+
+                    }catch(Exception e){
+                        return;
+                    }
+
+                    mapset.clear();
+                    mapnotset.clear();
+
+                    mapset.applyPattern(cc.map,false);
+                    mapnotset.applyPattern(cc.mapnot,false);
+
+                    charset.getUnicodeSet(unicodeset, cc.which);
+                    UnicodeSet diffset = new UnicodeSet();
+
+                    //are there items that must be in unicodeset but are not?
+                    (diffset = mapset).removeAll(unicodeset);
+                    if(!diffset.isEmpty()){
+                        StringBuffer s = new StringBuffer(diffset.toPattern(true));
+                        if(s.length()>100){
+                            s.replace(0, 0x7fffffff, ellipsis);
+                        }
+                        errln("error in missing items - conversion/getUnicodeSet test case "+cc.charset + "\n" + s.toString());
+                    }
+
+                    //are the items that must not be in unicodeset but are?
+                    (diffset=mapnotset).retainAll(unicodeset);
+                    if(!diffset.isEmpty()){
+                        StringBuffer s = new StringBuffer(diffset.toPattern(true));
+                        if(s.length()>100){
+                            s.replace(0, 0x7fffffff, ellipsis);
+                        }
+                        errln("contains unexpected items - conversion/getUnicodeSet test case "+cc.charset + "\n" + s.toString());
+                    }
+        } catch (Exception e) {
+            errln("getUnicodeSet returned an error code");
+            errln("ErrorCode expected is: " + cc.outErrorCode);
+            errln("Error Result is: " + e.toString());
+            return;
+        }
     }
 
     /**
      * This follows ucnv.c method ucnv_detectUnicodeSignature() to detect the
      * start of the stream for example U+FEFF (the Unicode BOM/signature
      * character) that can be ignored.
-     * 
+     *
      * Detects Unicode signature byte sequences at the start of the byte stream
      * and returns number of bytes of the BOM of the indicated Unicode charset.
      * 0 is returned when no Unicode signature is recognized.
-     * 
+     *
      */
 
     private String detectUnicodeSignature(ByteBuffer source) {
@@ -1051,10 +1095,12 @@ public class TestConversion extends ModuleTest {
         expected.rewind();
         output.limit(output.position());
         output.rewind();
-        
+
         // remove any BOM signature before checking
-        detectUnicodeSignature(output); // sets the position to after the BOM
-        output = output.slice(); // removes anything before the current position
+        if (!cc.charset.contains("UnicodeLittle") && !cc.charset.contains("UnicodeBig")) {
+            detectUnicodeSignature(output); // sets the position to after the BOM
+            output = output.slice(); // removes anything before the current position
+        }
 
         if (output.limit() != expected.limit()) {
             errln("Test failed: output length does not match expected for charset: " + cc.charset
@@ -1070,7 +1116,7 @@ public class TestConversion extends ModuleTest {
                 }
             }
         }
-        
+
         if (res) {
             logln("[" + cc.caseNr + "]:" + cc.charset);
             logln("Input:       " + printchars(CharBuffer.wrap(cc.unicode), cc.unicode.length()));
@@ -1096,11 +1142,7 @@ public class TestConversion extends ModuleTest {
 
         // test to see if the conversion matches actual results
         if (output.limit() != expected.length()) {
-            if (skipIfBeforeICU(4,5,0)) { // TIME BOMB
-                logln("Skipping test:(" + cc.charset + ") due to time bomb");
-            } else {
-                errln("Test failed: output length does not match expected for charset: "+cc.charset+ " [" + cc.caseNr + "]");
-            }
+            errln("Test failed: output length does not match expected for charset: "+cc.charset+ " [" + cc.caseNr + "]");
             res = false;
         } else {
             for (int i = 0; i < expected.length(); i++) {
@@ -1112,16 +1154,13 @@ public class TestConversion extends ModuleTest {
                 }
             }
         }
-        
+
         if (res) {
             logln("[" + cc.caseNr + "]:" + cc.charset);
             logln("Input:       " + printbytes(cc.bytes, cc.bytes.limit()));
             logln("Output:      " + printchars(output, output.limit()));
             logln("Expected:    " + printchars(CharBuffer.wrap(expected), expected.length()));
             logln("Passed");
-        }
-        else if (skipIfBeforeICU(4,5,0)) {
-            // TIME BOMB
         } else {
             errln("[" + cc.caseNr + "]:" + cc.charset);
             errln("Input:       " + printbytes(cc.bytes, cc.bytes.limit()));
