@@ -1,3 +1,5 @@
+// © 2016 and later: Unicode, Inc. and others.
+// License & terms of use: http://www.unicode.org/copyright.html
 /*
 ******************************************************************************
 * Copyright (C) 2007, International Business Machines Corporation and        *
@@ -9,23 +11,12 @@
 
 package com.ibm.icu.dev.test.duration.languages;
 
-import com.ibm.icu.dev.test.duration.LanguageTestRoot;
+import com.ibm.icu.dev.test.duration.LanguageTestFmwk;
 
 /**
  * Test cases for es
  */
-public class Test_es extends LanguageTestRoot {
-
-  /**
-   * Invoke the tests.
-   */
-  public static void main(String[] args) {
-      new Test_es().run(args);
-  }
-
-  /**
-   * Constructor.
-   */
+public class Test_es extends LanguageTestFmwk {
   public Test_es() {
     super("es", false);
   }
